#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>

#define TCOUNT 15
#define COUNT_LIMIT 6

int     count = 0;
pthread_mutex_t count_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t count_threshold_cv = PTHREAD_COND_INITIALIZER;

void *inc_count() 
{
  int j,i;
  double result=0.0;

  for (i=0; i < TCOUNT; i++) {
    
    pthread_mutex_lock(&count_mutex);
    printf("Producer locked mutex and incrementing count\n");

    count++;

    /* 
    Check the value of count and signal waiting thread when condition is
    reached.  Note that this occurs while mutex is locked. 
    */
    if (count == COUNT_LIMIT) {
      printf("inc_count(): producer reached the threshold count = %d.\n", count);
      printf("Producer sending the signal\n");
      pthread_cond_signal(&count_threshold_cv);
    }

     printf("Producer unlocking the mutex\n");
     pthread_mutex_unlock(&count_mutex);
    
    /* Do some work so threads can alternate on mutex lock */
     for (j=0; j < 25000; j++)
      result = result + (double)random();
   }
  pthread_exit(NULL);
}

void *watch_count() 
{

  printf("Starting consumer thread.\n");

  /*
  Lock mutex and wait for signal.  Note that the pthread_cond_wait routine
  will automatically and atomically unlock mutex while it waits. 
  Also, note that if COUNT_LIMIT is reached before this routine is run by
  the waiting thread, the loop will be skipped to prevent pthread_cond_wait
  from never returning.
  */
  pthread_mutex_lock(&count_mutex);
  printf("Consumer locked the mutex\n");
  
  if (count < COUNT_LIMIT) {
    pthread_cond_wait(&count_threshold_cv, &count_mutex);
    printf("watch_count(): consumer received condition signal.\n");
    count += 1000;
    printf("Consumer incremented count by thousand: %d\n", count);
  }
  printf("Consumer unclocking the mutex\n");
  pthread_mutex_unlock(&count_mutex);
  pthread_exit(NULL);
}

int main(int argc, char *argv[])
{
  int i, rc;
  pthread_t producer, consumer;
  
  pthread_create(&producer, NULL, inc_count, NULL);
  pthread_create(&consumer, NULL, watch_count, NULL);

  pthread_join(producer, NULL);
  pthread_join(consumer, NULL);

  /* Clean up and exit */
  pthread_mutex_destroy(&count_mutex);
  pthread_cond_destroy(&count_threshold_cv);
  
  exit(0);
}

